function [ProjX,Vec,Off,Val] = reduceusingpca(x,reduction)
% REDUCEUSINGPCA Finds a lower dimensional representation of the dataset
% using Principal Component Analysis.
% 18th Machine Learning Summer School
%
% Author: Ruben Martinez-Cantin <rmcantin@unizar.es>
%
% Output:
%   ProjX = projected data
%   Vec = sorted vectors of the principal components
%   Off = mean of data
%   Val = sorted values of the principal components
%
% Usage:
%   1) No reduction provided. Project the dataset x to the PCA space, but
%   no dimensionality reduction is provided.
%    [p,v] = reduceusingpca(x);
%
%   2) 0 < reduction < 1. Project the dataset x to the PCA space. It
%   selects the principal components that guarantee that "reduction"
%   proportion of the original information is preserved 
%    [p,v] = reduceusingpca(x,0.75);    % Projects to a lower dimensional 
%                                       % space that preserves 75% of data
%
%   3) reduction > 1. Project the dataset x to the PCA space. It
%   selects the number of principal components that can are selected. 
%    [p,v] = reduceusingpca(x,20);      % Projects to a 20 dimensional
%                                       % space

Off=mean(x);                % find mean of input matrix
y=x-ones(size(x,1),1)*Off;  % normalise by subtracting mean
c=cov(y);                   % find covariance matrix

[V,D]=eig(c);               % find eigenvectors (V) and eigenvalues (D) of covariance matrix
[D,idx] = sort(diag(D));    % sort eigenvalues in descending order by first diagonalising eigenvalue matrix, 
                            % idx stores order to use when ordering eigenvectors
D = D(end:-1:1)';
V = V(:,idx(end:-1:1));     % put eigenvectors in order to correspond with eigenvalues

% Significant Principal Components we use.
if  (nargin == 1) || (reduction <= 0)
    index = 1:length(D);
elseif (reduction < 1)
    cumD = cumsum(D)./sum(D);
    index = cumD<reduction;
else
    index = 1:reduction;           
end;
Vec=V(:,index);
Val=D(:,index);
prefinal=Vec'*y';
ProjX=prefinal';            % final is normalised data projected onto eigenspace